/**
 * @file m13_ra6m3_ek_eeprom.c
 * @brief M13-RA6M3-EK eeprom BSP
 *
 * @section License
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Copyright (C) 2020-2021 M13DESIGN SARL. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author M13DESIGN SARL (www.m13design.fr)
 * @version 1.0.0
 **/

// Depencencies
#include "m13_ra6m3_ek_status.h"
#include "m13_ra6m3_ek_eeprom.h"
#include "m13_ra6m3_ek_vcom.h"
#include "m13_ra6m3_ek_i2c.h"
#include "string.h"
#include "debug.h"

// Macro definitions
#define ACK_PAGE_WRITE_DELAY 120000

// Global variables
bsp_i2c_handle_t eeprom_handle = BSP_I2C_INVALID_HANDLE;

/**
 * @name       bsp_eeprom_init
 * @brief      Initialize and open I2C channel 0
 * @param[in]  None
 * @return     value : ADC value
 **/
bsp_status_t bsp_eeprom_init(void)
{
    // Retrieve I2C channel 0 handle
    eeprom_handle = bsp_i2c_open(BSP_I2C_CHANNEL_0);

    if (eeprom_handle == BSP_I2C_INVALID_HANDLE)
    {
        return BSP_STATUS_INVALID_CHANNEL;
    }

    PRINTF_INFO(" EEPROM ready! \r\n");
}

/**
 * @name       bsp_eeprom_deninit
 * @brief      Initialize and open I2C channel 0
 * @param[in]  None
 * @return     value : ADC value
 **/
bsp_status_t bsp_eeprom_deninit(void)
{
    return 0;
}

/**
 * @name       bsp_eeprom_write
 * @brief      Write data to EEPROM
 * @param[in]  addr : Starting address
 * @param[in]  data : Pointer to the buffer containing the data to be written to the EEPROM
 * @param[in]  size : Size of the data to be written
 * @return     bsp status error code
 **/
bsp_status_t bsp_eeprom_write(uint16_t addr, const void *data, uint32_t size)
{
    bsp_status_t error = BSP_STATUS_OK;
    uint32_t write_size;

    uint32_t write_addr = (uint32_t)addr;
    const uint8_t *p_data;
    p_data = (const uint8_t *) data;

    while (size > 0)
    {
      	write_size = EEPROM_PAGE_WRITE_BUF - (write_addr % EEPROM_PAGE_WRITE_BUF);

    	if(size < write_size)
    		write_size = size;

        // EEPROM Page Write
        error = bsp_i2c_write (eeprom_handle, (BSP_I2C_SLAVE_ADDR_EEPROM + (write_addr/256) ), (write_addr % 256), 1, p_data, write_size);

        if (error != BSP_STATUS_OK)
        {
            return error;
        }
        else
        {
            // Update data pointer, size information
            p_data += write_size;
            write_addr += write_size;
            size -= write_size;

            // Wait until Page Write operation returns ACK
            bsp_eeprom_write_delay();
        }
    }

    return (error);
}

/**
 * @name       bsp_eeprom_read
 * @brief      Read data from EEPROM
 * @param[in]  addr : Starting address
 * @param[in]  data : Pointer to the buffer receiving the data to be read to the EEPROM
 * @param[in]  size : Size of the data to be read
 * @return     bsp status error code
 **/
bsp_status_t bsp_eeprom_read (uint16_t addr, void *data, uint32_t size)
{
	uint32_t read_size;
	uint32_t read_addr = (uint32_t)addr;
	bsp_status_t error = BSP_STATUS_OK;
    uint8_t *p_data;
    uint32_t size_Read;
    p_data = (uint8_t *)data;
    size_Read = size;

    while(size > 0)
    {

    	read_size = 256 - (read_addr % 256);

		if(size < read_size)
			read_size = size;

		// EEPROM sequential read
        error = bsp_i2c_read (eeprom_handle, ( BSP_I2C_SLAVE_ADDR_EEPROM + (read_addr/256) ), ( read_addr % 256 ), 1, p_data, read_size );

		size -= read_size;
		p_data += read_size;
		read_addr += read_size;
    }

    return (error);
}

/**
 * @name       bsp_eeprom_dump
 * @brief      Read all the data from EEPROM and displays it in the console (VCOM)
 * @param[in]  None
 * @return     bsp status error code
 **/
bsp_status_t bsp_eeprom_dump(void)
{
    bsp_status_t error;
	uint8_t i;
	static uint8_t data[EEPROM_MAX_SIZE];

	PRINTF_INFO("\r\n EEPROM Dump Start\r\n");

	//Read register contents
	error = bsp_eeprom_read(i, &data, sizeof(data) );
	//Any error to report?

	// Display EEPROM data in the console (VCOM)
	TRACE_INFO_ARRAY("  ", data, sizeof(data));

	PRINTF_INFO(" EEPROM Dump End\r\n");


	return error;
}

/**
 * @name       bsp_eeprom_test
 * @brief      This function will test if the EEPROM can be written and read correctly. (For test and debug purpose only)
 * @param[in]  None
 * @return     bsp status error code
 **/
bsp_status_t bsp_eeprom_test(void)
{
    int16_t i;
    int16_t n;
    static int8_t data[2048];
    bsp_status_t error = BSP_STATUS_OK;
    uint8_t ref[] = "AZERTY_QSDFGH_15463543641363645143651164354654136.54163.54136.514hjg65g4.63f54jhg316hj54hg1635hk6j413.h5j436hj.3k1jhk4hjg0k541kjhg3";

    PRINTF_INFO(" EEPROM test start ! \r\n");

    // Clear data buffer
    memset(data, 0x00, sizeof(data));

    // Clear eeprom with data buffer
    error = bsp_eeprom_write(0x00, data, sizeof(data));

    //PRINTF_INFO("\r\n Dump 2 - after eeprom clear \r\n");
    //bsp_eeprom_dump();

    strcpy(data, ref);
    n = strlen(data);

    error = bsp_eeprom_write(15, data, n);

    //PRINTF_INFO("\r\n Dump 2 - after write \r\n");
    //bsp_eeprom_dump();

    memset(data, 0x00, sizeof(data));

    error= bsp_eeprom_read(15, data, n);

    if(memcmp(data, ref, n) == 0)
    {
        PRINTF_INFO(" EEPROM test succeeded ! \r\n");
    }
    else
    {
        PRINTF_INFO(" EEPROM test failed ! \r\n");
        error = BSP_STATUS_EEPROM_FAILURE;
    }

    return (error);
}

/**
 * @name       bsp_eeprom_write_delay
 * @brief      Manual delay to wait for the ACK during page write operation
 * @param[in]  None
 * @return     None
 **/
void bsp_eeprom_write_delay(void)
{
    uint32_t delay = ACK_PAGE_WRITE_DELAY;

    // TODO : Measure and adjust the delay timing
    // Manual delay until the full Page Write operation ACK is returned (16 bytes)
    while(delay--)
    {
        __asm__ ("nop");
    }
}
