/**
 * @file m13_ra6m3_ek_led.c
 * @brief M13-RA6M3-EK led BSP
 *
 * @section License
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Copyright (C) 2020-2021 M13DESIGN SARL. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author M13DESIGN SARL (www.m13design.fr)
 * @version 1.0.0
 **/

#include "bsp_api.h"
#include "common_data.h"
#include "m13_ra6m3_ek_vcom.h"
#include "m13_ra6m3_ek_pinout.h"
#include "m13_ra6m3_ek_i2c.h"
#include "FreeRTOS.h"

// Global variables
extern SemaphoreHandle_t g_i2c0_semaphore;
extern SemaphoreHandle_t g_sci6_semaphore;

// I2C Callback functions
void i2c_master_callback(i2c_master_callback_args_t *p_args);
void sci_i2c_master_callback(i2c_master_callback_args_t *p_args);

// I2C IOPORT structure
static ioport_instance_ctrl_t i2c_pin_ctrl;

// I2C0 instance
iic_master_instance_ctrl_t i2c_channel_0_ctrl;

// I2C0 extended configuration
const iic_master_extended_cfg_t i2c_channel_0_extend =
{
  .timeout_mode             = IIC_MASTER_TIMEOUT_MODE_SHORT,
  .clock_settings.brl_value = 15,
  .clock_settings.brh_value = 16,
  .clock_settings.cks_value = 4,
};

// I2C0 channel configuration
const i2c_master_cfg_t i2c_channel_0_cfg =
{
     .channel        = 0,
     .rate           = I2C_MASTER_RATE_STANDARD,
     .slave          = BSP_I2C_SLAVE_ADDR_NONE,
     .addr_mode      = I2C_MASTER_ADDR_MODE_7BIT,
     .p_transfer_tx  = NULL,
     .p_transfer_rx  = NULL,
     .p_callback     = i2c_master_callback,
     .p_context      = NULL,
     .rxi_irq        = VECTOR_NUMBER_IIC0_RXI,
     .txi_irq        = VECTOR_NUMBER_IIC0_TXI,
     .tei_irq        = VECTOR_NUMBER_IIC0_TEI,
     .eri_irq        = VECTOR_NUMBER_IIC0_ERI,
     .ipl            = (12),
     .p_extend       = &i2c_channel_0_extend,
};

// I2C0 interface module
const i2c_master_instance_t i2c_channel_0 =
{
     .p_ctrl         = &i2c_channel_0_ctrl,
     .p_cfg          = &i2c_channel_0_cfg,
     .p_api          = &g_i2c_master_on_iic
};

// SCI6 instance
sci_i2c_instance_ctrl_t sci_channel_6_ctrl;

// SCI6 extended configuration
const sci_i2c_extended_cfg_t sci_channel_6_cfg_extend =
{
    .clock_settings.clk_divisor_value   = 0,
    .clock_settings.brr_value           = 22,
    .clock_settings.mddr_value          = 157,
    .clock_settings.bitrate_modulation  = true,
    .clock_settings.cycles_value        = 31,
    .clock_settings.snfr_value          = (1),
};

// SCI6 channel configuration
const i2c_master_cfg_t sci_channel_6_cfg =
{
     .channel       = 6,
     .rate          = I2C_MASTER_RATE_STANDARD,
     .slave         = BSP_I2C_SLAVE_ADDR_NONE,
     .addr_mode     = I2C_MASTER_ADDR_MODE_7BIT,
     .p_transfer_tx = NULL,
     .p_transfer_rx = NULL,
     .p_callback    = sci_i2c_master_callback,
     .p_context     = NULL,
     .rxi_irq       = FSP_INVALID_VECTOR,
     .txi_irq       = VECTOR_NUMBER_SCI6_TXI,
     .tei_irq       = VECTOR_NUMBER_SCI6_TEI,
     .ipl           = (12),
     .p_extend      = &sci_channel_6_cfg_extend,
};

// SCI6 interface module
const i2c_master_instance_t sci_channel_6 =
{
     .p_ctrl = &sci_channel_6_ctrl,
     .p_cfg = &sci_channel_6_cfg,
     .p_api = &g_i2c_master_on_sci
};

/**
 * @name       i2c_master_callback
 * @brief      Callback for I2C channel module
 * @param[in]  *p_args : I2C master callback data
 * @return     None
 */
void i2c_master_callback(i2c_master_callback_args_t *p_args)
{
    FSP_PARAMETER_NOT_USED(p_args);
    BaseType_t context_switch;

    /* Set Vsync semaphore */
    xSemaphoreGiveFromISR(g_i2c0_semaphore, &context_switch);

    portYIELD_FROM_ISR(context_switch);
}

/**
 * @name       sci_i2c_master_callback
 * @brief      Callback for SCI I2C channel module
 * @param[in]  *p_args : I2C master callback data
 * @return     None
 */
void sci_i2c_master_callback(i2c_master_callback_args_t *p_args)
{
    FSP_PARAMETER_NOT_USED(p_args);
    BaseType_t context_switch;

    /* Set Vsync semaphore */
    xSemaphoreGiveFromISR(g_sci6_semaphore, &context_switch);

    portYIELD_FROM_ISR(context_switch);
}

/**
 * @name       i2c_wait
 * @brief      Function to wait for I2C communication completion
 * @param[in]  None
 * @return     None
 */
static void i2c_wait()
{
    xSemaphoreTake(g_i2c0_semaphore, portMAX_DELAY);
}

/**
 * @name       sci_wait
 * @brief      Function to wait for SCI/I2C communication completion
 * @param[in]  None
 * @return     None
 */
static void sci_wait()
{
    xSemaphoreTake(g_sci6_semaphore, portMAX_DELAY);
}

/**
 * @name       bsp_i2c_init
 * @brief      Configure I2C pins
 * @param[in]  channel : I2C channel number
 * @return     bsp_status_t
 */
bsp_status_t bsp_i2c_init(bsp_i2c_channel_t channel)
{
    ioport_cfg_t i2c_pin_cfg;

    if (channel == BSP_I2C_CHANNEL_0)
    {
        i2c_pin_cfg.number_of_pins = sizeof(m13_ra6m3_ek_i2c0_pin_data)/sizeof(ioport_pin_cfg_t);
        i2c_pin_cfg.p_pin_cfg_data = &m13_ra6m3_ek_i2c0_pin_data[0];

        //Configure led pins
        R_IOPORT_Open(&i2c_pin_ctrl, &i2c_pin_cfg);
    }
    else if (channel == BSP_I2C_CHANNEL_6)
    {
        i2c_pin_cfg.number_of_pins = sizeof(m13_ra6m3_ek_sci6_pin_data)/sizeof(ioport_pin_cfg_t);
        i2c_pin_cfg.p_pin_cfg_data = &m13_ra6m3_ek_sci6_pin_data[0];

        //Configure led pins
        R_IOPORT_Open(&i2c_pin_ctrl, &i2c_pin_cfg);
    }
    else
    {
        return BSP_STATUS_INIT_FAILURE;
    }

    return BSP_STATUS_OK;
}

/**
 * @name       bsp_i2c_open
 * @brief      Configure I2C pins
 * @param[in]  bsp_i2c_channel_t : I2C channel number
 * @return     bsp_status_t
 */
bsp_i2c_handle_t bsp_i2c_open(bsp_i2c_channel_t channel)
{
    fsp_err_t error;
    bsp_i2c_handle_t *handle = NULL;

    if(channel == BSP_I2C_CHANNEL_0)
    {
        error = R_IIC_MASTER_Open(&i2c_channel_0_ctrl, &i2c_channel_0_cfg);

        if(error == FSP_SUCCESS)
            handle = (void*) &i2c_channel_0_ctrl;
    }
    else if(channel == BSP_I2C_CHANNEL_6)
    {
        error = R_SCI_I2C_Open(&sci_channel_6_ctrl, &sci_channel_6_cfg);

        if(error == FSP_SUCCESS)
            handle = (void*) &sci_channel_6_ctrl;
    }
    else
    {
        handle = NULL;
    }

    return handle;
}


/**
 * @name       bsp_i2c_write
 * @brief      Perform an i2c write access
 * @param[in]  handle : Handle for RZ/A2M device control function
 * @param[in]  device_address : I2C slave address (7-bit value)
 * @param[in]  sub_address : Device's register address
 * @param[in]  Sub_address_length : Register address length
 * @param[in]  *data : Data to be written
 * @param[in]  data_length : Data length
 * @return     error code
 */
bsp_status_t bsp_i2c_write(bsp_i2c_handle_t handle, uint8_t device_address, uint32_t sub_address,
    uint32_t sub_address_length, const uint8_t *data, size_t data_length)
{
    fsp_err_t error;
    size_t i;
    size_t buffer_length = sub_address_length + data_length;
    uint8_t buffer[buffer_length];
    i = 0;

    // Calculate Check sub-address length and copy it into the buffer
    if(sub_address_length == 1)
    {
        buffer[i++] = sub_address & 0xFF;
    }
    else if (sub_address_length == 2)
    {
        buffer[i++] = (sub_address >> 8 ) & 0xFF;
        buffer[i++] = sub_address & 0xFF;
    }
    else if (sub_address_length == 4)
    {
        buffer[i++] = (sub_address >> 24 ) & 0xFF;
        buffer[i++] = (sub_address >> 16 ) & 0xFF;
        buffer[i++] = (sub_address >> 8 ) & 0xFF;
        buffer[i++] = sub_address & 0xFF;
    }
    else
    {
        return BSP_STATUS_INVALID_PARAMETER;
    }

    // Add the data to the buffer
    memcpy(buffer + i, data, data_length);

    // Check if the opened handler is r_iic or r_sci and call/use the appropriate renesas api for the write operation
    if ( handle == &i2c_channel_0_ctrl )
    {
        // Set I2C slave device_address
        i2c_channel_0_ctrl.slave = (device_address);

        error = R_IIC_MASTER_Write(handle, buffer, buffer_length, false);

        i2c_wait();

        if ( error != FSP_SUCCESS )
        {
            return BSP_STATUS_I2C_WRITE_FAILURE;
        }
    }
    else if ( handle == &sci_channel_6_ctrl )
    {
        // Set I2C slave device_address
        sci_channel_6_ctrl.slave = (device_address);

        error = R_SCI_I2C_Write(handle, buffer, buffer_length, false);

        sci_wait();

        if ( error != FSP_SUCCESS )
        {
            return BSP_STATUS_I2C_WRITE_FAILURE;
        }
    }
    else
    {
        PRINTF_INFO("Error I2C \r\n");
        return BSP_STATUS_I2C_WRITE_FAILURE;
    }

    if ( error != FSP_SUCCESS )
    {
        return BSP_STATUS_I2C_WRITE_FAILURE;
    }
    else
    {
        return BSP_STATUS_OK;
    }
}

/**
 * @name       bsp_i2c_read
 * @brief      Perform an i2c read access
 * @param[in]  handle : Handle for RZ/A2M device control function
 * @param[in]  device_address : I2C slave address (7-bit value)
 * @param[in]  sub_address : Device's register address
 * @param[in]  Sub_address_length : Register address length
 * @param[in]  *data : Data table to retreive data read
 * @param[in]  data_length : Data length
 * @return     error code
 */
bsp_status_t bsp_i2c_read(bsp_i2c_handle_t handle, uint8_t device_address, uint32_t sub_address,
    uint32_t sub_address_length, uint8_t *data, size_t data_length)
{
    fsp_err_t error;
    uint8_t buffer[4];

    // Check sub-address length and format the sub_address data accordingly
    if(sub_address_length == 1)
    {
        buffer[0] = sub_address & 0xFF;
    }
    else if (sub_address_length == 2)
    {
        buffer[0] = (sub_address >> 8 ) & 0xFF;
        buffer[1] = sub_address & 0xFF;
    }
    else if (sub_address_length == 4)
    {
        buffer[0] = (sub_address >> 24 ) & 0xFF;
        buffer[1] = (sub_address >> 16 ) & 0xFF;
        buffer[2] = (sub_address >> 8 ) & 0xFF;
        buffer[3] = sub_address & 0xFF;
    }
    else
    {
        return BSP_STATUS_INVALID_PARAMETER;
    }

    // Check if the opened handler is r_iic or r_sci and call/use the appropriate renesas api for the read operation
    if ( handle == &i2c_channel_0_ctrl )
    {
        // Set I2C slave device_address
        i2c_channel_0_ctrl.slave = (device_address);

        error = R_IIC_MASTER_Write(handle, &buffer[0], sub_address_length, true);

        i2c_wait();

        if ( error != FSP_SUCCESS )
        {
            return BSP_STATUS_I2C_READ_FAILURE;
        }

        error = R_IIC_MASTER_Read(handle, data, data_length, false);

        i2c_wait();

        if ( error != FSP_SUCCESS )
        {
            return BSP_STATUS_I2C_READ_FAILURE;
        }

    }
    else if ( handle == &sci_channel_6_ctrl )
    {
        // Set I2C slave device_address
        sci_channel_6_ctrl.slave = (device_address);

        error = R_SCI_I2C_Write(handle, &buffer[0], sub_address_length, true);

        sci_wait();

        if ( error != FSP_SUCCESS )
        {
            return BSP_STATUS_I2C_READ_FAILURE;
        }

        error = R_SCI_I2C_Read(handle, data, data_length, false);

        sci_wait();

        if ( error != FSP_SUCCESS )
        {
            return BSP_STATUS_I2C_READ_FAILURE;
        }
    }
    else
    {
        PRINTF_INFO("Error I2C \r\n");
        return BSP_STATUS_I2C_READ_FAILURE;
    }

    return BSP_STATUS_OK;
}

