

// Dependencies
#include "bsp_api.h"
#include "m13_ra6m3_ek_sdram.h"
#include "m13_ra6m3_ek_pinout.h"
#include "r_ioport.h"


// Macro definitions

// Set ACTIVE-to-PRECHARGE command (tRAS) timing
// e.g. tRAS = 42ns -> 6cycles are needed at SDCLK 120MHz
//      tRAS = 37ns -> 5cycles are needed at SDCLK 120MHz
 
#define BSP_PRV_SDRAM_TRAS                  (6)

// Set ACTIVE-to-READ or WRITE delay tRCD (tRCD) timing
// e.g. tRCD = 18ns -> 3cycles are needed at SDCLK 120MHz
//      tRCD = 15ns -> 2cycles are needed at SDCLK 120MHz

#define BSP_PRV_SDRAM_TRCD                  (3)


// Set PRECHARGE command period (tRP) timing
// e.g. tRP  = 18ns -> 3cycles are needed at SDCLK 120MHz
//      tRP  = 15ns -> 2cycles are needed at SDCLK 120MHz

#define BSP_PRV_SDRAM_TRP                   (3)


// Set WRITE recovery time (tWR) timing
// e.g. tWR  = 1CLK + 6ns -> 2cycles are needed at SDCLK 120MHz
//      tWR  = 1CLK + 7ns -> 2cycles are needed at SDCLK 120MHz

#define BSP_PRV_SDRAM_TWR                   (2)

// Set CAS (READ) latency (CL) timing
// e.g. CL = 18ns -> 3cycles are needed at SDCLK 120MHz
// e.g. CL = 15ns -> 2cycles are needed at SDCLK 120MHz

#define BSP_PRV_SDRAM_CL                    (3)

// Set AUTO REFRESH period (tRFC) timing
// e.g. tRFC = 60nS -> 8cycles are needed at SDCLK 120MHz
//      tRFC = 66nS -> 8cycles are needed at SDCLK 120MHz

#define BSP_PRV_SDRAM_TRFC                  (8)

// Set Average Refresh period
// e.g. tREF = 64ms/8192rows -> 7.8125us/each row.  937cycles are needed at SDCLK 120MHz

#define BSP_PRV_SDRAM_REF_CMD_INTERVAL      (937)

// Set Auto-Refresh issue times in initialization sequence needed for SDRAM device
// Typical SDR SDRAM device needs twice of Auto-Refresh command issue

#define BSP_PRV_SDRAM_SDIR_REF_TIMES        (2)

// Set RAW address offset
// Available settings are
// 8  : 8-bit
// 9  : 9-bit
// 10 : 10-bit
// 11 : 11-bit

#define BSP_PRV_SDRAM_SDADR_ROW_ADDR_OFFSET (9)

// Select endian mode for SDRAM address space
// 0 : Endian of SDRAM address space is the same as the endian of operating mode
// 1 : Endian of SDRAM address space is not the endian of operating mode

#define BSP_PRV_SDRAM_ENDIAN_MODE           (0)

// Select access mode
// Typically Continuous access should be enabled to get better SDRAM bandwidth
// 0: Continuous access is disabled
// 1: Continuous access is enabled

#define BSP_PRV_SDRAM_CONTINUOUS_ACCESSMODE (1)

// Select bus width
// 0: 16-bit
// 2: 8-bit

#define BSP_PRV_SDRAM_BUS_WIDTH             (0)

#if ((BSP_PRV_SDRAM_SDADR_ROW_ADDR_OFFSET != 8) && (BSP_PRV_SDRAM_SDADR_ROW_ADDR_OFFSET != 9) \
  && (BSP_PRV_SDRAM_SDADR_ROW_ADDR_OFFSET != 10) && (BSP_PRV_SDRAM_SDADR_ROW_ADDR_OFFSET > 11))
#error "BSP_PRV_SDRAM_SDADR_ROW_ADDR_OFFSET must be either of 8,9,10 or 11"
#endif

#if ((BSP_PRV_SDRAM_BUS_WIDTH != 0) && (BSP_PRV_SDRAM_BUS_WIDTH != 2))
#error "BSP_PRV_SDRAM_BUS_WIDTH must be either of 0(16-bit) or 2(8-bit)"
#endif

#if ((BSP_PRV_SDRAM_ENDIAN_MODE != 0) && (BSP_PRV_SDRAM_ENDIAN_MODE != 1))
#error "BSP_PRV_SDRAM_ENDIAN_MODE must be either of 0(same endian as operating mode) or 2(another endian against operating mode)"
#endif

#if ((BSP_PRV_SDRAM_CONTINUOUS_ACCESSMODE != 0) && (BSP_PRV_SDRAM_CONTINUOUS_ACCESSMODE != 1))
#error "BSP_PRV_SDRAM_CONTINUOUS_ACCESSMODE must be either of 0(continuous access is disabled) or 1(continuous access is enabled)"
#endif

#define BSP_PRV_SDRAM_MR_WB_SINGLE_LOC_ACC  (1)     /* MR.M9                : Single Location Access */
#define BSP_PRV_SDRAM_MR_OP_MODE            (0)     /* MR.M8:M7             : Standard Operation */
#define BSP_PRV_SDRAM_MR_BT_SEQUENCTIAL     (0)     /* MR.M3 Burst Type     : Sequential */
#define BSP_PRV_SDRAM_MR_BURST_LENGTH       (0)     /* MR.M2:M0 Burst Length: 0(1 burst) */

// Typedef definitions

// Private function prototypes

// Private global variables
static ioport_instance_ctrl_t sdram_pin_ctrl;


void bsp_sdram_io_init (void)
{
    ioport_cfg_t sdram_pin_cfg;

    sdram_pin_cfg.number_of_pins = sizeof(m13_ra6m3_ek_sdram_pin_data)/sizeof(ioport_pin_cfg_t);

    sdram_pin_cfg.p_pin_cfg_data = &m13_ra6m3_ek_sdram_pin_data[0];

    R_IOPORT_Open(&sdram_pin_ctrl, &sdram_pin_cfg);
}

void bsp_sdram_init (void)
{

    // Configure the sdram pins
    bsp_sdram_io_init();

    // Delay at least 100uS after SDCLK active
    R_BSP_SoftwareDelay(100, BSP_DELAY_UNITS_MICROSECONDS);

    /** Setting for SDRAM initialization sequence */
#if (BSP_PRV_SDRAM_TRP < 3)
    R_BUS->SDIR_b.PRC  = 0;
#else
    //R_BUS->SDIR_b.PRC  = BSP_PRV_SDRAM_TRP - 3;
    R_BUS->SDRAM.SDIR_b.PRC = BSP_PRV_SDRAM_TRP - 3;
#endif

    //while(R_BUS->SDSR)
    while(R_BUS->SDRAM.SDSR)
    {
        // According to h/w manual, need to confirm that all the status bits in SDSR are 0 before SDIR modification
    }

    R_BUS->SDRAM.SDIR_b.ARFC = BSP_PRV_SDRAM_SDIR_REF_TIMES;

    while(R_BUS->SDRAM.SDSR)
    {
        // According to h/w manual, need to confirm that all the status bits in SDSR are 0 before SDIR modification
    }

#if (BSP_PRV_SDRAM_TRFC < 3)
    R_BUS->SDIR_b.ARFI = 0;
#else
    R_BUS->SDRAM.SDIR_b.ARFI = BSP_PRV_SDRAM_TRFC - 3;
#endif

    while(R_BUS->SDRAM.SDSR)
    {
        // According to h/w manual, need to confirm that all the status bits in SDSR are 0 before SDIR modification
    }

    /** Start SDRAM initialization sequence.
     * Following operation is automatically done when set SDICR.INIRQ bit.
     * Perform a PRECHARGE ALL command and wait at least tRP time.
     * Issue an AUTO REFRESH command and wait at least tRFC time.
     * Issue an AUTO REFRESH command and wait at least tRFC time.
     */
    R_BUS->SDRAM.SDICR_b.INIRQ = 1;
    while(R_BUS->SDRAM.SDSR_b.INIST)
    {
        // Wait the end of initialization sequence
    }

    /** Setting for SDRAM controller */
    R_BUS->SDRAM.SDCCR_b.BSIZE  = BSP_PRV_SDRAM_BUS_WIDTH;              /* set SDRAM bus width */
    R_BUS->SDRAM.SDAMOD_b.BE    = BSP_PRV_SDRAM_CONTINUOUS_ACCESSMODE;  /* enable continuous access */
    R_BUS->SDRAM.SDCMOD_b.EMODE = BSP_PRV_SDRAM_ENDIAN_MODE;            /* set endian mode for SDRAM address space */

    while(R_BUS->SDRAM.SDSR)
    {
        /* According to h/w maual, need to confirm that all the status bits in SDSR are 0 before SDMOD modification. */
    }

    /** Using LMR command, program the mode register */
    R_BUS->SDRAM.SDMOD = ((((BSP_PRV_SDRAM_MR_WB_SINGLE_LOC_ACC << 9)
                   |(BSP_PRV_SDRAM_MR_OP_MODE           << 7))
                   |(BSP_PRV_SDRAM_CL                   << 4))
                   |(BSP_PRV_SDRAM_MR_BT_SEQUENCTIAL    << 3))
                   |(BSP_PRV_SDRAM_MR_BURST_LENGTH      << 0);

    /** wait at least tMRD time */
    while(R_BUS->SDRAM.SDSR_b.MRSST)
    {
        /* Wait until Mode Register setting done. */
    }

    /** Set timing parameters for SDRAM */
    R_BUS->SDRAM.SDTR_b.RAS = BSP_PRV_SDRAM_TRAS - 1;       /* set ACTIVE-to-PRECHARGE command cycles*/
    R_BUS->SDRAM.SDTR_b.RCD = BSP_PRV_SDRAM_TRCD - 1;       /* set ACTIVE to READ/WRITE delay cycles */
    R_BUS->SDRAM.SDTR_b.RP  = BSP_PRV_SDRAM_TRP  - 1;       /* set PRECHARGE command period cycles */
    R_BUS->SDRAM.SDTR_b.WR  = BSP_PRV_SDRAM_TWR  - 1;       /* set write recovery cycles */
    R_BUS->SDRAM.SDTR_b.CL  = BSP_PRV_SDRAM_CL;             /* set SDRAM column latency cycles */

    /** Set row address offset for target SDRAM */
    R_BUS->SDRAM.SDADR_b.MXC   = BSP_PRV_SDRAM_SDADR_ROW_ADDR_OFFSET - 8;

    R_BUS->SDRAM.SDRFCR_b.REFW = BSP_PRV_SDRAM_TRFC - 1;                /* set Auto-Refresh issuing cycle */
    R_BUS->SDRAM.SDRFCR_b.RFC  = BSP_PRV_SDRAM_REF_CMD_INTERVAL - 1;    /* set Auto-Refresh period */

    /** Start Auto-refresh */
    R_BUS->SDRAM.SDRFEN_b.RFEN = 1;

    /** Enable SDRAM access */
    R_BUS->SDRAM.SDCCR_b.EXENB = 1;
}


